# Solfege - free ear training software
# Copyright (C) 2007, 2008, 2011, 2016 Tom Cato Amundsen
# License is GPL, see file COPYING


import unittest


class TestSourceCode(unittest.TestCase):
    pass
suite = unittest.makeSuite(TestSourceCode)
