rem GNU Solfege - free eartraining
rem Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2013  Tom Cato Amundsen

rem set PATH=..\mypython\Lib\site-packages\gtk
set PATH=win32\python\Lib\site-packages\gtk


rem win32\python\python.exe win32-start-solfege.pyw srcdir
..\mypython\python.exe win32-start-solfege.pyw srcdir

pause
